package com.jkcredit.traffic.record.util;

import com.jkcredit.traffic.record.model.MonthResult;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SparkSession;

import java.util.List;

/**
 * @author xusonglin
 * @version V1.0
 **/

public class SparkTool {

    public static String appName = "DataExtract";
    private static JavaSparkContext jsc = null;
    private static SparkSession spark = null;

    private static void initSpark() {
        if (jsc == null || spark == null) {

            SparkConf sparkConf = new SparkConf();
            sparkConf.set("spark.driver.allowMultipleContexts", "true");
            sparkConf.set("spark.eventLog.enabled", "true");
            sparkConf.set("spark.serializer", "org.apache.spark.serializer.KryoSerializer");
            sparkConf.set("spark.hadoop.validateOutputSpecs", "false");
            sparkConf.set("hive.mapred.supports.subdirectories", "true");
            sparkConf.set("mapreduce.input.fileinputformat.input.dir.recursive", "true");

            spark = SparkSession.builder().appName(appName).master("local").config(sparkConf).getOrCreate();
            jsc = new JavaSparkContext(spark.sparkContext());
        }

    }

    public static JavaSparkContext getJsc() {
        if (jsc == null) {
            initSpark();
        }
        return jsc;
    }

    public static SparkSession getSession() {
        if (spark == null) {
            initSpark();
        }
        return spark;

    }


    public static void main(String[] args) {
        getSession();
        /**
         * val monthDataset: Dataset[monthResult] = spark.read.option("header", true).csv(month_path).toDF("month", "vehicleid",
         *       "max_exTime", "sum_travel_time", "max_travel_time", "sum_feemileage", "sum_fee", "sum_weight_mileage",
         *       "exTimes_count", "axlecount", "vehicletype", "travel_provinces_count", "transtime_count").as[monthResult].coalesce(36)
         */
        Dataset<MonthResult> monthResultDataset = spark
                .read()
                .option("header", true)
                .csv("/Users/jkxy/Desktop/test/part-00000-4b3bccbf-a8ad-4bfe-b9a4-31e455238c9c-c000.csv")
                .toDF("month", "vehicleid", "max_exTime", "sum_travel_time", "max_travel_time", "sum_feemileage",
                        "sum_fee", "sum_weight_mileage", "exTimes_count", "axlecount", "vehicletype",
                        "travel_provinces_count", "transtime_count")
                .as(Encoders.bean(MonthResult.class))
                .coalesce(36);
        monthResultDataset.createOrReplaceTempView("monthDataset");

        // 110
        Dataset<MonthResult> rows = spark.sql("select * from monthDataset where sum_travel_time < 2678400 " +
                "and max_travel_time < 604800 and sum_feemileage < 89280000 and sum_weight_mileage < 89280000 and exTimes_count < 1200")
                .as(Encoders.bean(MonthResult.class));
        List<MonthResult> results = rows.javaRDD().collect();
        spark.sqlContext().dropTempTable("monthDataset");
        spark.stop();
    }
}
